'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Theme = require('../Theme');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var styles = {
  wrapper: {
    position: 'relative',
    width: '100%',
    paddingBottom: '100%',
    overflow: 'hidden'
  },
  contentBox: {
    position: 'absolute',
    top: 0,
    bottom: 0,
    left: 0,
    right: 0
  }
};

var getRatioValue = function getRatioValue(ratio) {
  var res = '100%';
  var vals = void 0;
  try {
    vals = ratio.match(/\d+/g); // 16/9 => [16, 9]
  } catch (err) {
    return res;
  }
  res = 100 * vals[1] / vals[0];
  return res;
};

var getWrapperStyles = function getWrapperStyles(_ref) {
  var ratio = _ref.ratio;
  return _extends({}, styles.wrapper, {
    paddingBottom: ratio ? getRatioValue(ratio) + '%' : '100%'
  });
};

var getBorderStyle = function getBorderStyle(hasBorder) {
  return hasBorder ? {
    border: _Theme.theme.border.default,
    borderRadius: '2px'
  } : {};
};

/**
 * the purpose of this compo is to provide a box (DOM div)
 * that enforce an apsect ratio
 * default to square (1/1), accept a custom ratio (e.g. '16/9', '2/1' ...)
 */
var RatioBox = function RatioBox(_ref2) {
  var children = _ref2.children,
      ratio = _ref2.ratio,
      style = _ref2.style,
      hasBorder = _ref2.hasBorder;
  return _react2.default.createElement(
    'div',
    { style: _extends({}, getWrapperStyles({ ratio: ratio }), getBorderStyle(hasBorder), style) },
    _react2.default.createElement(
      'div',
      { style: styles.contentBox },
      children
    )
  );
};

exports.default = RatioBox;