'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDataGridHeader = exports.createDataGridBody = exports.createDataGridColumn = exports.createDataGridCell = undefined;

var _templateObject = _taggedTemplateLiteral(['\n  position: absolute;\n  right: 12px;\n  top: 0;\n  bottom: 0;\n'], ['\n  position: absolute;\n  right: 12px;\n  top: 0;\n  bottom: 0;\n']);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

var _Table = require('../../Table');

var _TableHeaderCheckedAllCell = require('../../Table/TableHeaderCheckedAllCell');

var _TableHeaderCheckedAllCell2 = _interopRequireDefault(_TableHeaderCheckedAllCell);

var _DataGridSorting = require('./DataGridSorting');

var _DataGridSorting2 = _interopRequireDefault(_DataGridSorting);

var _ActionMenu = require('../../ActionMenu');

var _ActionMenu2 = _interopRequireDefault(_ActionMenu);

var _Text = require('../../Text');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _taggedTemplateLiteral(strings, raw) { return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

var ActionMenuWrapper = _styledComponents2.default.div(_templateObject);

var createDataGridCell = exports.createDataGridCell = function createDataGridCell(property, actions, viewModel, keyFromPropertyOrIndex) {
  if (!actions) {
    return _react2.default.createElement(
      _Table.TableRowColumn,
      {
        hasAction: !!actions,
        key: keyFromPropertyOrIndex,
        'data-key': keyFromPropertyOrIndex
      },
      property
    );
  }

  return _react2.default.createElement(
    _Table.TableRowColumn,
    {
      hasAction: !!actions,
      key: keyFromPropertyOrIndex,
      'data-key': keyFromPropertyOrIndex
    },
    _react2.default.createElement(
      'div',
      null,
      property,
      actions && _react2.default.createElement(
        ActionMenuWrapper,
        { className: 'actionMenuInTableRowColumn' },
        _react2.default.createElement(_ActionMenu2.default, {
          value: viewModel.original,
          menuDescriptors: actions,
          mainMenuButtonItemStyle: { height: '50px' }
        })
      )
    )
  );
};

// that's arow not acolumn, rioght ?
var createDataGridColumn = exports.createDataGridColumn = function createDataGridColumn(viewModel, actions, index) {
  var rowKey = viewModel.id || viewModel.key || index;
  return _react2.default.createElement(
    _Table.TableRow,
    {
      hasAction: !!actions,
      key: rowKey,
      'data-key': viewModel.id || viewModel.key || index,
      value: viewModel.key
    },
    viewModel.properties.map(function (property, idx) {
      var keyFromPropertyOrIndex = viewModel && viewModel.propertiesDefinitions && viewModel.propertiesDefinitions[idx] && viewModel.propertiesDefinitions[idx].name || idx;

      var cellKey = rowKey + '-' + keyFromPropertyOrIndex;

      return createDataGridCell(property, idx === 0 ? actions : null, viewModel, cellKey);
    })
  );
};

var createDataGridBody = exports.createDataGridBody = function createDataGridBody(viewModels, isHidden, actions) {
  var result = [];

  viewModels.forEach(function (viewModel, index) {
    result.push(createDataGridColumn(viewModel, actions, index));
  });

  return _react2.default.createElement(
    _Table.TableBody,
    { style: isHidden ? { visibility: 'hidden' } : {}, viewModels: viewModels },
    result
  );
};

var createDataGridHeader = exports.createDataGridHeader = function createDataGridHeader() {
  var headers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var fixedHeight = arguments[1];
  var hideHeader = arguments[2];
  var stickyHeader = arguments[3];
  var withCheckbox = arguments[4];
  var checkAllHandler = arguments[5];
  var allRowsSelected = arguments[6];
  return _react2.default.createElement(
    _Table.TableHeader,
    { stickyHeader: stickyHeader },
    _react2.default.createElement(
      _Table.TableRow,
      { hideSvg: hideHeader },
      withCheckbox && _react2.default.createElement(_TableHeaderCheckedAllCell2.default, {
        dataKey: 'TableHeaderCheckedAllCell',
        allRowsSelected: allRowsSelected,
        onCheckAll: checkAllHandler
      }),
      headers.map(function (header, i) {
        var key = 'header-' + (header.name || header.displayName || i);
        if (header.menuItems) {
          return _react2.default.createElement(_DataGridSorting2.default, {
            menuItems: header.menuItems,
            dataKey: key,
            style: hideHeader ? { visibility: 'hidden' } : {},
            title: header.displayName
          });
        }

        if (header.extraMenuItems) {
          return _react2.default.createElement(_DataGridSorting2.default, {
            extraMenuItems: header.extraMenuItems,
            dataKey: key,
            style: hideHeader ? { visibility: 'hidden' } : {},
            title: header.displayName
          });
        }

        if (header.isSortable) {
          return _react2.default.createElement(_DataGridSorting2.default, {
            dataKey: key,
            style: hideHeader ? { visibility: 'hidden' } : {},
            title: header.displayName
          });
        }

        // default (not sortable) ——————————
        return _react2.default.createElement(
          _Table.TableHeaderColumn,
          {
            key: key,
            style: hideHeader ? { visibility: 'hidden' } : {}
          },
          _react2.default.createElement(
            _Text.TextEllipsis,
            null,
            header.displayName
          )
        );
      })
    )
  );
};

exports.default = {
  createDataGridHeader: createDataGridHeader,
  createDataGridBody: createDataGridBody,
  createDataGridColumn: createDataGridColumn,
  createDataGridCell: createDataGridCell
};