'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toViewModel = exports.entityToViewModel = exports.toHeaderDefinition = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DefaultCellComponent = function DefaultCellComponent(_ref) {
  var value = _ref.value;

  if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
    return JSON.stringify(value);
  }

  return value || '';
};

var toHeaderDefinition = exports.toHeaderDefinition = function toHeaderDefinition(data, properties) {
  var headers = properties || Object.keys(data[0] || {}) || [];

  return headers.map(function (h) {
    if ((typeof h === 'undefined' ? 'undefined' : _typeof(h)) === 'object') {
      return h;
    }
    return {
      displayName: h
    };
  });
};

var createPropertyViewModel = function createPropertyViewModel(entity, properties) {
  var keys = properties || Object.keys(entity);
  // here we can play with the system to find the correct Component
  // we can play with the Type and many other things

  return keys.map(function (k) {
    return {
      name: k.name,
      propertyName: k.property ? k.property : k,
      isComputed: k.isComputed,
      Component: k.Component ? k.Component : DefaultCellComponent
    };
  });
};

var createPropertiesWithValue = function createPropertiesWithValue(propertiesDefinitions, entity, getTypeDefinition) {
  return propertiesDefinitions.map(function (propertyDefinition) {
    var name = propertyDefinition.propertyName;
    var entityValue = entity[name];
    var PropertyComponent = propertyDefinition.Component;
    var Converter = getTypeDefinition && getTypeDefinition(entityValue);

    if (propertyDefinition.isComputed) {
      return _react2.default.createElement(PropertyComponent, { entity: entity });
    }

    if (Converter) {
      return _react2.default.createElement(Converter, { value: entityValue });
    }

    return _react2.default.createElement(PropertyComponent, { value: entityValue });
  });
};

var getEntityKey = function getEntityKey(entity) {
  var propertyKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  return entity[propertyKey] || entity.key || entity.id || entity.Id;
};

var entityToViewModel = exports.entityToViewModel = function entityToViewModel(entity, propertiesDefinitions, propertyKey, getTypeDefinition) {
  return {
    id: entity.id || entity.Id || null,
    key: getEntityKey(entity, propertyKey),
    original: entity,
    properties: createPropertiesWithValue(propertiesDefinitions, entity, getTypeDefinition),
    propertiesDefinitions: propertiesDefinitions
  };
};

/**
 * return an Array of PropertyViewModel
 * [
 *  {
 *    key: 'name',
 *    original : 'orignal value'
 *    properties: [
 *      <ReactNode/>,
 *      <ReactNode/>
 *    ]
 *  }
 * ]
 */
var toViewModel = exports.toViewModel = function toViewModel() {
  var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var properties = arguments[1];
  var propertyKey = arguments[2];
  var getTypeDefinition = arguments[3];

  var propertiesDefinitions = createPropertyViewModel(data[0], properties); // same entity

  return data.map(function (entity) {
    return entityToViewModel(entity, propertiesDefinitions, propertyKey, getTypeDefinition);
  });
};

exports.default = {
  toHeaderDefinition: toHeaderDefinition,
  toViewModel: toViewModel
};