'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PropsMapperContainerQueries = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

var _resizeObserverPolyfill = require('resize-observer-polyfill');

var _resizeObserverPolyfill2 = _interopRequireDefault(_resizeObserverPolyfill);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Mobile first (intended to work only with minWidth)
 *
 * const myPropsMapper = props => { return (do stuff) };
 * const myPropsMapper = ({ shouldTooltipBeVisible }) => { return (do stuff) };
 *
 * const rules = [{
 *   minWidth: 700, // number
 *   propsMapper: myPropsMapper,
 * }]
 *
 * <PropsContainerQuery rules={rules} />
 */

var applyRules = function applyRules(props, rules, width, height) {
  if (!rules || rules.length === 0) {
    return props;
  }

  var result = _extends({}, props);

  var internalRules = rules
  // .filter(rule => 'minWidth' in rule)  // TODO improve
  /**
   * we only apply the mapper if the available width
   * is wider or equal than the 'minWidth' condition
   * if you order you rules correcty you get a nice
   * logical, ascending, flow of overwrite
   * !!!avoid cyclic dependent constraint!!
   */
  .filter(function (_ref) {
    var minWidth = _ref.minWidth;
    return !width || width >= minWidth;
  });

  internalRules.forEach(function (_ref2) {
    var mapper = _ref2.mapper;

    result = _extends({}, result, mapper(_extends({}, result, { containerWidth: width, containerHeight: height })));
  });

  return result;
};

var PropsMapperContainerQueries = exports.PropsMapperContainerQueries = function (_Component) {
  _inherits(PropsMapperContainerQueries, _Component);

  function PropsMapperContainerQueries(props) {
    _classCallCheck(this, PropsMapperContainerQueries);

    var _this = _possibleConstructorReturn(this, (PropsMapperContainerQueries.__proto__ || Object.getPrototypeOf(PropsMapperContainerQueries)).call(this, props));

    _this.init = _this.init.bind(_this);
    _this.storeRef = _this.storeRef.bind(_this);

    _this.ref = null;
    _this.observer = null;

    _this.state = {
      mappedProps: null
    };
    return _this;
  }

  _createClass(PropsMapperContainerQueries, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (window && window.addEventListener) {
        window.addEventListener('resize', this.handleResize);
      }
      this.init();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (window && window.removeEventListener) {
        window.removeEventListener('resize', this.handleResize);
      }
      if (this.observer) {
        this.observer.disconnect();
      }
    }
  }, {
    key: 'storeRef',
    value: function storeRef(node) {
      this.ref = node;
      this.init();
    }
  }, {
    key: 'init',
    value: function init() {
      var _this2 = this;

      var _props = this.props,
          rules = _props.rules,
          _props$children = _props.children,
          children = _props$children === undefined ? {} : _props$children;


      if (this.observer) {
        // noop watch on unmount
      } else if (this.ref) {
        this.observer = new _resizeObserverPolyfill2.default(function (entries /* , observer */) {
          var last = entries[entries.length - 1];

          var _last$contentRect = last.contentRect,
              width = _last$contentRect.width,
              height = _last$contentRect.height;


          var mappedProps = applyRules(_extends({}, _this2.props, children.props || {}), rules, width, height);
          if (!(0, _isEqual2.default)(_this2.state.mappedProps, mappedProps)) {
            _this2.setState({ mappedProps: mappedProps });
          }
        });

        this.observer.observe(this.ref);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _props2 = this.props,
          children = _props2.children,
          inline = _props2.inline,
          style = _props2.style;
      var mappedProps = this.state.mappedProps;


      var type = inline ? 'span' : 'div';

      try {
        _react2.default.Children.only(children);
      } catch (err) {
        console.error('PropsMapperContainerQueries accept a single child, see this SO https://goo.gl/2sF2eb');
        return null;
      }

      var extendedChildren = _react2.default.cloneElement(children, _extends({}, mappedProps || {}, {
        rules: undefined
      }));

      return _react2.default.createElement(type, {
        ref: this.storeRef,
        style: _extends({}, style || {}, this.ref && mappedProps ? {} : { opacity: 0 })
      }, extendedChildren);
    }
  }]);

  return PropsMapperContainerQueries;
}(_react.Component);

PropsMapperContainerQueries.propTypes = {};
exports.default = PropsMapperContainerQueries;