'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PropsMapperMediaQueries = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _debounce = require('lodash/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Mobile first (intended to work only with minWidth)
 *
 * const myPropsMapper = props => { return (do stuff) };
 * const myPropsMapper = ({ shouldTooltipBeVisible }) => { return (do stuff) };
 *
 * const rules = [{
 *   minWidth: 700, // number
 *   mapper: myPropsMapper,
 * }]
 *
 * <PropsContainerQuery rules={rules} />
 */

var applyRules = function applyRules(props, rules, width, height) {
  if (rules.length === 0) {
    return props;
  }

  var result = _extends({}, props);

  // let internalRules = rules.filter(rule => 'minWidth' in rule); // TODO improve
  /**
     * we only apply the mapper if the available
     * WINDOW/VIEWPORT width
     * is wider or equal than the 'minWidth' condition
     * if you order you rules correcty you get a nice
     * logical, ascending, flow of overwrite
     * Mobile first FTW
     */
  var internalRules = rules.filter(function (_ref) {
    var minWidth = _ref.minWidth,
        minHeight = _ref.minHeight;
    return width >= minWidth || height >= minHeight;
  });

  if (internalRules.length === 0) {
    return props;
  }

  internalRules.forEach(function (_ref2) {
    var mapper = _ref2.mapper;

    result = _extends({}, result, mapper(props));
  });

  return result;
};

var PropsMapperMediaQueries = exports.PropsMapperMediaQueries = function (_Component) {
  _inherits(PropsMapperMediaQueries, _Component);

  function PropsMapperMediaQueries(props) {
    _classCallCheck(this, PropsMapperMediaQueries);

    var _this = _possibleConstructorReturn(this, (PropsMapperMediaQueries.__proto__ || Object.getPrototypeOf(PropsMapperMediaQueries)).call(this, props));

    var debounceDelay = _this.props.debounceDelay;


    _this.handleResize = (0, _debounce2.default)(_this.handleResize.bind(_this), debounceDelay).bind(_this);

    _this.state = {
      mappedProps: null
    };
    return _this;
  }

  _createClass(PropsMapperMediaQueries, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (window) {
        window.addEventListener('resize', this.handleResize);
        this.forceUpdate();
      }
      this.handleResize();
      this.forceUpdate();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (window) {
        window.removeEventListener('resize', this.handleResize);
      }
    }
  }, {
    key: 'handleResize',
    value: function handleResize() {
      if (window) {
        var _window = window,
            height = _window.innerHeight,
            width = _window.innerWidth;
        var _props = this.props,
            rules = _props.rules,
            _props$children = _props.children,
            children = _props$children === undefined ? {} : _props$children;

        var mappedProps = applyRules(_extends({}, this.props, children.props || {}), rules, width, height);

        if (!(0, _isEqual2.default)(this.state.mappedProps, mappedProps)) {
          var _rules = mappedProps.rules,
              restOfMappedProps = _objectWithoutProperties(mappedProps, ['rules']);

          this.setState({
            mappedProps: restOfMappedProps
          });
        }
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var children = this.props.children;
      var mappedProps = this.state.mappedProps;


      var theChild = _react2.default.Children.only(children);

      return _react2.default.cloneElement(theChild, _extends({}, mappedProps));
    }
  }]);

  return PropsMapperMediaQueries;
}(_react.Component);

PropsMapperMediaQueries.propTypes = {
  rules: _propTypes2.default.array,
  children: _propTypes2.default.node,
  debounceDelay: _propTypes2.default.number
};
PropsMapperMediaQueries.defaultProps = {
  rules: [],
  children: null,
  debounceDelay: 180
};
exports.default = PropsMapperMediaQueries;